@@ -1,4 +1,4 @@
-# pylint: disable=missing-docstring, invalid-name, too-few-public-methods, no-self-use
+# pylint: disable=missing-docstring, invalid-name, too-few-public-methods, no-self-use, useless-object-inheritance
 
 def test_regression_737():
     import xml # [unused-variable]
@@ -29,3 +29,22 @@ class HasUnusedDunderClass(object):
 
     def best(self):
         self.test()
+
+
+def locals_example_defined_before():
+    value = 42  # [possibly-unused-variable]
+    return locals()
+
+
+def locals_example_defined_after():
+    local_variables = locals()
+    value = 42  # [unused-variable]
+    return local_variables
+
+
+def locals_does_not_account_for_subscopes():
+    value = 42  # [unused-variable]
+
+    def some_other_scope():
+        return locals()
+    return some_other_scope
